unit sc_form;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Cabinet, Fci;

type
  TMainForm = class(TForm)
    GroupBox1: TGroupBox;
    CompressBtn: TButton;
    Label1: TLabel;
    FilesEdit: TEdit;
    NameEdit: TEdit;
    Label2: TLabel;
    GroupBox2: TGroupBox;
    Label3: TLabel;
    ExtractBtn: TButton;
    DestEdit: TEdit;
    BetterCheckBox: TCheckBox;
    CabinetWriter1: TCabinetWriter;
    CabinetReader1: TCabinetReader;
    StreamCabinetFileHandler1: TStreamCabinetFileHandler;
    ListBox1: TListBox;
    StatusBar1: TStatusBar;
    procedure CompressBtnClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure ExtractBtnClick(Sender: TObject);
    procedure CabinetReader1CopyFile(Sender: TObject;
      const FileName: String; UncompressedSize: Integer; Date, Time,
      Attribs: Smallint; var Action: TFileCopyAction;
      var DestFileHandle: Integer);
  private
    { Private declarations }
    procedure Status (const Text : string);
    procedure Log (const Text : string);
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

uses
  Utility;

{$R *.DFM}

procedure TMainForm.Status (const Text : string);
begin
  StatusBar1.SimpleText:=Text;
  StatusBar1.Update;
end;

procedure TMainForm.Log (const Text : string);
begin
  ListBox1.ItemIndex:=ListBox1.Items.Add(Text);
  ListBox1.Update;
end;

procedure TMainForm.CompressBtnClick(Sender: TObject);
var
  SRec : TSearchRec;
  Fn, Spec : string;
  Compr : TCompressionParameters;
begin
  ListBox1.Items.Clear;
  Status('Compressing ' + NameEdit.Text);
  try
    Log('Preparing...');

    Spec:=FilesEdit.Text;
    if BetterCheckBox.Checked then
      Compr:=MakeLzxCompression(21)
    else
      Compr:=MakeMsZipCompression;

    CabinetWriter1.Open(NameEdit.Text, '', MaxInt, 900000, 0);
    if FindFirst(Spec, faAnyFile, SRec) = 0 then
      repeat
        if (SRec.Name <> '.') and (SRec.Name <> '..') and
          (SRec.Attr and faDirectory = 0) then
        begin
          Fn:=AddBkSlash(ExtractFilePath(Spec)) + SRec.Name;
          Log('Adding ' + Fn + '...');
          CabinetWriter1.AddFile(Fn, SRec.Name, [], Compr);
        end;
      until FindNext(SRec) <> 0;
    FindClose(SRec);
                    
    Log('Flushing cabinet');
    CabinetWriter1.FlushCabinet(True);
    Log('Closing cabinet');
    CabinetWriter1.Close;
  finally
    Status('Done');
  end;
end;

procedure TMainForm.FormActivate(Sender: TObject);
begin
  Refresh;
end;

procedure TMainForm.ExtractBtnClick(Sender: TObject);
begin
  ListBox1.Items.Clear;
  Status('Extracting');
  try
    Log('Extracting files...');
    CabinetReader1.ExtractFiles(NameEdit.Text, DestEdit.Text, 0);
  finally
    Status('Done');
  end;
end;

procedure TMainForm.CabinetReader1CopyFile(Sender: TObject;
  const FileName: String; UncompressedSize: Integer; Date, Time,
  Attribs: Smallint; var Action: TFileCopyAction;
  var DestFileHandle: Integer);
begin
  Log('Extracting ' + FileName + ' to ' +
    (Sender as TCabinetReader).DestinationPath + '...');        

  Action:=fcaDefaultCopy;    // Let component handle the copying
end;

end.
